import time

t=time.time()
def needle(a, b, gapopen, gapextend, endgapopen, endgapextend):
    def E_FPEQ(a, b, e): return (((b - e) < a) & (a < (b + e)))

    sub = [[5, -4, -4, -4, -4, 1, 1, -4, -4, 1, -4, -1, -1, -1, -2, -4], [-4, 5, -4, -4, -4, 1, -4, 1, 1, -4, -1, -4, -1, -1, -2, 5], [-4, -4, 5, -4, 1, -4, 1, -4, 1, -4, -1, -1, -4, -1, -2, -4], [-4, -4, -4, 5, 1, -4, -4, 1, -4, 1, -1, -1, -1, -4, -2, -4], [-4, -4, 1, 1, -1, -4, -2, -2, -2, -2, -1, -1, -3, -3, -1, -4], [1, 1, -4, -4, -4, -1, -2, -2, -2, -2, -3, -3, -1, -1, -1, 1], [1, -4, 1, -4, -2, -2, -1, -4, -2, -2, -3, -1, -3, -1, -1, -4], [-4, 1, -4, 1, -2, -2, -4, -1, -2, -2, -1, -3, -1, -3, -1, 1], [-4, 1, 1, -4, -2, -2, -2, -2, -1, -4, -1, -3, -3, -1, -1, 1], [1, -4, -4, 1, -2, -2, -2, -2, -4, -1, -3, -1, -1, -3, -1, -4], [-4, -1, -1, -1, -1, -3, -3, -1, -1, -3, -1, -2, -2, -2, -1, -1], [-1, -4, -1, -1, -1, -3, -1, -3, -3, -1, -2, -1, -2, -2, -1, -4], [-1, -1, -4, -1, -3, -1, -3, -1, -3, -1, -2, -2, -1, -2, -1, -1], [-1, -1, -1, -4, -3, -1, -1, -3, -1, -3, -2, -2, -2, -1, -1, -1], [-2, -2, -2, -2, -1,-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -2], [-4, 5, -4, -4, -4, 1, -4, 1, 1, -4, -1, -4, -1, -1, -2, 5]]; # EDNAFUL

    base_to_idx = {'A': 0, 'C': 3, 'B': 10, 'D': 13, 'G': 2, 'H': 12, 'K': 8, 'M': 9, 'N': 14, 'S': 4, 'R': 6, 'U': 15, 'T': 1, 'W': 5, 'V': 11, 'Y': 7}; # for EDNAFULL
    idx_to_base = {0: 'A', 1: 'T', 2: 'G', 3: 'C', 4: 'S', 5: 'W', 6: 'R', 7: 'Y', 8: 'K', 9: 'M', 10: 'B', 11: 'V', 12: 'H', 13: 'D', 14: 'N', 15: 'U'}; # for EDNAFULL

    ix = {}
    iy = {}
    m = {}

    compass = {}
    p = []
    q = []

    lena = len(a)
    lenb = len(b)

    
    for i in range(lena): p.append(base_to_idx[a[i]])
    for i in range(lenb): q.append(base_to_idx[b[i]])

    match = sub[p[0]][q[0]]
    
    aln_a = []
    aln_b = []
    aln_r = []

    ix[0] = -endgapopen-gapopen
    iy[0] = -endgapopen-gapopen
    m[0] = match
    
    for ypos in range(1, lena):
        match = sub[p[ypos]][q[0]]
        cursor = ypos * lenb
        cursorp = (ypos - 1) * lenb

        testog = m[cursorp] - gapopen
        testeg = iy[cursorp] - gapopen

        if testog >= testeg: iy[cursor] = testog
        else: iy[cursor] = testeg

        m[cursor] = match - (endgapopen + (ypos - 1) * endgapextend)
        ix[cursor] = -endgapopen - ypos * endgapextend - gapopen

    ix[cursor] -= endgapopen
    ix[cursor] += gapopen
    
    cursor = 0

    for xpos in range(1, lenb):
        match = sub[p[0]][q[xpos]]
        cursor = xpos
        cursorp = xpos - 1

        testog = m[cursorp] - gapopen
        testeg = ix[cursorp] - gapextend
        
        if testog >= testeg: ix[cursor] = testog
        else: ix[cursor] = testeg

        m[cursor] = match - (endgapopen + (xpos - 1) * endgapextend)
        iy[cursor] = -endgapopen - xpos * endgapextend - gapopen

    iy[cursor] -= endgapopen
    iy[cursor] += gapopen

    xpos = 1
    
    while xpos != lenb:
        ypos = 1
        bconvcode = q[xpos]
        
        cursorp = xpos - 1
        cursor = xpos

        xpos+=1

        while (ypos < lena):

            match = sub[p[ypos]][bconvcode]
            ypos += 1
            cursor += lenb

            mp = m[cursorp]
            ixp = ix[cursorp]
            iyp = iy[cursorp]

            if (mp > ixp and mp > iyp): m[cursor] = mp + match
            elif ixp > iyp: m[cursor] = ixp + match
            else: m[cursor] = iyp + match

            cursorp += 1

            if xpos == lenb:
                testog = m[cursorp] - endgapopen
                testeg = iy[cursorp] - endgapextend
            else:
                testog = m[cursorp]
                if testog < ix[cursorp]: testog = ix[cursorp]

                testog -= gapopen
                testeg = iy[cursorp] - gapextend

            if testog > testeg: iy[cursor] = testog
            else: iy[cursor] = testeg
            
            cursorp += lenb

            cursorp -= 1

            if ypos == lena:
                testog = m[cursorp] - endgapopen
                testeg = ix[cursorp] -endgapextend
            else:
                testog = m[cursorp]
                if testeg < iy[cursorp]: testog = iy[cursorp]
                testog -= gapopen
                testeg = ix[cursorp] - gapextend
            if testog > testeg: ix[cursor] = testog
            else: ix[cursor] =testeg
        
    score = -32767 # INT MIN
    start1 = lena - 1
    start2 = lenb - 1

    cursor = lena * lenb - 1
    if m[cursor] > ix[cursor] and m[cursor] > iy[cursor]: score = m[cursor]
    elif ix[cursor] > iy[cursor]: score = ix[cursor]
    else: score = iy[cursor]

    cursorp = 0
    cursor = 1

    eps = 1.192e-6

    ypos = start1
    xpos = start2
    


    while xpos >= 0 and ypos >= 0:
        cursor = ypos * lenb + xpos
        mp = m[cursor]

        if ypos == 0 or ypos == lena - 1: FPEQ_a1 = endgapextend
        else: FPEQ_a1 = gapextend
        
        if xpos == 0 or xpos == lenb - 1: FPEQ_a2 = endgapextend
        else: FPEQ_a2 = gapextend

        if cursorp == 1 and E_FPEQ(FPEQ_a1, ix[cursor]-ix[cursor+1], eps):
            compass[cursor] = 1
            xpos -= 1
        elif cursorp == 2 and E_FPEQ(FPEQ_a2, iy[cursor]-iy[cursor+lenb], eps):
            compass[cursor] = 2
            ypos -= 1
        elif mp >= ix[cursor] and mp >= iy[cursor]:
            if cursorp == 1 and E_FPEQ(mp, ix[cursor], eps):
                compass[cursor] = 1
                xpos -= 1
            elif cursorp == 2 and E_FPEQ(mp, iy[cursor], eps):
                compass[cursor] = 2
                ypos -= 1
            else:
                compass[cursor] = 0
                ypos -= 1
                xpos -= 1
        elif ix[cursor] >= iy[cursor] and xpos > -1:
            compass[cursor] = 1
            xpos -= 1
        elif ypos > -1:
            compass[cursor] = 2
            ypos -= 1
        else:
            print("Needle: Something is seriously wrong in the traceback algorithm")
            return -1
        
        cursorp = compass[cursor]

    for i in range(lenb-1, start2, -1):
        aln_b.append(idx_to_base[q[i]])
        aln_a.append('-')
        aln_r.append(' ')
    for j in range(lena-1, start1, -1):
        aln_a.append(idx_to_base[p[j]])
        aln_b.append('-')
        aln_r.append(' ')

    while start2 >= 0 and start1 >= 0:
        cursor = start1 * lenb + start2
        if compass[cursor] == 0: # diagonal
            b1 = p[start1]
            b2 = q[start2]
            start1 -= 1
            start2 -= 1
            aln_a.append(idx_to_base[b1])
            aln_b.append(idx_to_base[b2])
            if b1 == b2: aln_r.append('|')
            else: aln_r.append('.')
            continue
        elif compass[cursor] == 1: # Left, gap(s) in vertical
            aln_a.append('-')
            aln_b.append(idx_to_base[q[start2]])
            aln_r.append(' ')
            start2 -= 1
            continue
        elif compass[cursor] == 2: # Down, gap(s) in horizontal
            aln_a.append(idx_to_base[p[start1]])
            aln_b.append('-')
            aln_r.append(' ')
            start1 -= 1
            continue
        else:
            print(compass)
            print("Needle: Walk Error in WN")
            return -1

    for i in range(start2, -1, -1):
        aln_b.append(idx_to_base[q[i]])
        aln_a.append('-')
        aln_r.append(' ')

    for i in range(start1, -1, -1):
        aln_a.append(idx_to_base[p[i]])
        aln_b.append('-')
        aln_r.append(' ')

    aln_a = "".join(aln_a[::-1])
    aln_b = "".join(aln_b[::-1])
    aln_r = "".join(aln_r[::-1])
    
    return [aln_a, aln_r, aln_b]



import argparse, time, sys, re
from subprocess import Popen, PIPE
from needle import needle

def rev_comp(s): return s.translate(s.maketrans('ATGCRYSWKMBDHV', 'TACGYRWSMKVHDB'))[::-1]

def rev_pos(rl, p): return rl - p - 1

def gc(s): return round((s.count('G') + s.count('C'))*100/len(s),2)

class PeDesigner:
    
    def __init__(self, args):
        ref_file = args.ref_directory
        ind_file = args.ind_directory
        sref = open(ref_file).readlines()
        sind = open(ind_file).readlines()
        if sref[0][0] != '>' or sind[0][0] !='>':
            print("[Error] the file is not FASTA format !!")
            sys.exit()
        self.ref_seq = ''.join(sref[1:]).replace('\n','').upper()
        self.ind_seq = ''.join(sind[1:]).replace('\n','').upper()
        self.pam = args.pam.upper()
        self.ref_dir = args.ref_dir
        self.rna_len = args.l
        self.mm_num = args.m + 1
        self.pbs_min = args.pbs_min
        self.pbs_max = args.pbs_max
        self.rtt_min = args.rtt_min
        self.rtt_max = args.rtt_max
        self.nick_min = args.nick_min
        self.nick_max = args.nick_max
        self.use_cpus = args.use_cpus
        self.seq_list = [] #d -> [seq, info_l, pbs_d, rtt_d, nick_d]
        self.off_d = {}

        self.offinder_input = [self.ref_dir, 'N'*self.rna_len + self.pam] 

    def set_mutation(self):
        needle_res = needle(self.ref_seq, self.ind_seq, 10, -2, 10, -2)
        self.mutation_st = -1
        for n, sym in enumerate(needle_res[1]):
            if sym != '|':
                print(n)
                self.mutation_st = n
                break
        self.mutation_ed = -1
        for n, sym in enumerate(needle_res[1][::-1]):
            if sym != '|':
                self.mutation_ed = len(needle_res[1]) - n
                break
        if self.mutation_st == -1 or self.mutation_ed == -1:
            print('reference sequence and induced sequence is same!!!')
            sys.exit()
        self.mutation_len = self.mutation_ed - self.mutation_st - 1
        self.needle_res = needle_res

    def find_target(self):
        pattern = 'N'*self.rna_len + self.pam
        pattern = '(' + pattern + ')'
        pattern = pattern.replace('N', '[AGTC]').replace('R', '[AG]').replace('W', '[AT]').replace('M', '[AC]').replace('Y', '[CT]').replace('S', '[GC]').replace('K', '[GT]').replace('B', '[CGT]').replace('D', '[AGT]').replace('H', '[ACT]').replace('V', '[ACG]')
        pattern = re.compile(pattern)
        pattern_r = '(' + rev_comp('N' * self.rna_len + self.pam) + ')'
        pattern_r = pattern_r.replace('N', '[AGTC]').replace('R', '[AG]').replace('W', '[AT]').replace('M', '[AC]').replace('Y', '[CT]').replace('S', '[GC]').replace('K', '[GT]').replace('B', '[CGT]').replace('D', '[AGT]').replace('H', '[ACT]').replace('V', '[ACG]')
        pattern_r = re.compile(pattern_r)
        pam_pattern = re.compile(self.pam.replace('N', '[AGTC]').replace('R', '[AG]').replace('W', '[AT]').replace('M', '[AC]').replace('Y', '[CT]').replace('S', '[GC]').replace('K', '[GT]').replace('B', '[CGT]').replace('D', '[AGT]').replace('H', '[ACT]').replace('V', '[ACG]'))

        pos = 0
        m = pattern.search(self.ref_seq, pos)
        targets = []
        while m != None:
            seq = m.group()
            pos = m.start() 
            if m.group(1) != None:
                pbs_l = []
                rtt_l = []
                minlen_rtt = self.mutation_ed + 1 - (pos + self.rna_len - 3 - 1) - self.needle_res[2].count('-')
                if minlen_rtt > self.rtt_max or pos + self.rna_len - 3 - 1 > self.mutation_st:
                    pos += 1
                    m = pattern.search(self.ref_seq, pos)
                    continue
                strand = '+'
                rel_pos = round((pos+(self.rna_len-3))*100/len(self.ref_seq),2)
                edit_pos = self.mutation_st - (pos + 16)
                if pam_pattern.search(self.ind_seq[pos+self.rna_len: pos + self.rna_len + len(self.pam)]) != None:
                    pam_change = 'No'
                else:
                    pam_change = 'Yes'
                for x in range(self.pbs_min, self.pbs_max + 1):
                    pbs_s = self.ref_seq[pos + self.rna_len - 3 - x: pos + self.rna_len - 3]
                    pbs_l.append([pbs_s, x, gc(pbs_s)])
                for x in range(self.rtt_min, self.rtt_max + 1):
                    if minlen_rtt > x:
                        continue
                    rtt_s =  self.ind_seq[pos + self.rna_len - 3: pos + self.rna_len -3 + x]
                    rtt_l.append([rtt_s, x, gc(rtt_s)])
                
                off_s = seq[:self.rna_len] + 'N' * len(self.pam) + ' ' + str(self.mm_num - 1) 
                if off_s not in self.offinder_input:
                    self.offinder_input.append(off_s)
                    self.off_d[seq[:self.rna_len]] = []
                    for x in range(self.mm_num):
                        self.off_d[seq[:self.rna_len]].append(0)
                
                sub_res = [seq, [pos + 1, rel_pos, strand, gc(seq[:self.rna_len]), edit_pos, pam_change], pbs_l, rtt_l, []]

                self.seq_list.append(sub_res)
            pos += 1
            m = pattern.search(self.ref_seq, pos)
        
        pos = 0 
        m = pattern_r.search(self.ref_seq, pos)
        while m != None:
            seq = m.group()
            pos = m.start()
            if m.group(1) != None:
                seq = rev_comp(seq)
                pbs_l = []
                rtt_l = []
                rp = rev_pos(len(self.ref_seq), (pos + self.rna_len + len(self.pam) - 1))
                minlen_rtt = rev_pos(len(self.needle_res[0]), self.mutation_st - 1) - (rp + self.rna_len - 3 - 1) - self.needle_res[2].count('-')
                if rev_pos(len(self.needle_res[0]), self.mutation_ed) < rp + self.rna_len - 3 - 1 or minlen_rtt > self.rtt_max:
                    pos += 1
                    m = pattern_r.search(self.ref_seq, pos)
                    continue
                strand = '-'
                rel_pos = round((pos+(3+len(self.pam)))*100/len(self.ref_seq),2)
                edit_pos = pos + self.needle_res[0].count('-')+ 6 - self.mutation_ed + 1
                if pam_pattern.search(rev_comp(self.ind_seq)[rp - self.rna_len - len(self.pam): rp - self.rna_len]) != None:
                    pam_change = 'No'
                else:
                    pam_change = 'Yes'
                for x in range(self.pbs_min, self.pbs_max + 1):
                    pbs_s = self.ref_seq[pos + len(self.pam) + 3: pos + len(self.pam) + 3 + x]
                    pbs_l.append([pbs_s, x, gc(pbs_s)])
                for x in range(self.rtt_min, self.rtt_max + 1):
                    if minlen_rtt > x:
                        continue
                    rtt_s = rev_comp(rev_comp(self.ind_seq)[rp + self.rna_len - 3: rp + self.rna_len - 3 + x])
                    rtt_l.append([rtt_s, x, gc(rtt_s)])
                
                off_s = seq[:self.rna_len] + 'N' * len(self.pam) + ' ' + str(self.mm_num - 1)
                if off_s not in self.offinder_input:
                    self.offinder_input.append(off_s)
                    self.off_d[seq[:self.rna_len]] = []
                    for x in range(self.mm_num):
                        self.off_d[seq[:self.rna_len]].append(0)
    
                sub_res = [seq, [pos + 1, rel_pos, strand, gc(seq[:self.rna_len]), edit_pos, pam_change], pbs_l, rtt_l, []]
    
                self.seq_list.append(sub_res)

            pos += 1
            m = pattern_r.search(self.ref_seq, pos)

        pos = 0
        m = pattern.search(self.ind_seq, pos)
        while m != None:
            seq = m.group()
            pos = m.start()
            selected = False
            PE_type = 'PE3'
            if pos + self.needle_res[2].count('-') < self.mutation_ed and pos + self.rna_len + len(self.pam) > self.mutation_st:
                PE_type= 'PE3b'
            if m.group(1) != None:
                rel_pos = round((pos + self.rna_len - 3 - 1) * 100 / len(self.ind_seq),2)
                for i in range(len(self.seq_list)):
                    line = self.seq_list[i]
                    if line[1][2] != '-':
                        continue
                    cv = line[1][0] + len(self.pam) + 3 - 1 + self.needle_res[0].count('-') - self.needle_res[2].count('-')
                    distance = (pos + self.rna_len - 3) - cv 
                    if distance == 0:
                        distance += 1
                    if self.nick_min <= abs(distance) <= self.nick_max:
                        sub_nick = [seq, pos + 1, rel_pos, '+', gc(seq[:self.rna_len]), distance, PE_type]
                        for x in range(self.mm_num):
                            sub_nick.append(0)
                        self.seq_list[i][4].append(sub_nick)
                        selected = True

            if selected == True:
                off_s = seq[:self.rna_len] + 'N' * len(self.pam) + ' ' + str(self.mm_num - 1)
                if off_s not in self.offinder_input:
                    self.offinder_input.append(off_s)
                    self.off_d[seq[:self.rna_len]] = []
                    for x in range(self.mm_num):
                        self.off_d[seq[:self.rna_len]].append(0)

            pos += 1
            m = pattern.search(self.ind_seq, pos)
        
        pos = 0
        m = pattern_r.search(self.ind_seq, pos)
        while m != None:
            seq = m.group()
            pos = m.start()
            selected = False
            PE_type = 'PE3'
            if pos + self.needle_res[2].count('-') < self.mutation_ed and pos + self.rna_len + len(self.pam) > self.mutation_st:
                PE_type= 'PE3b'
            if m.group(1) != None:
                rel_pos = round((pos + self.rna_len + len(self.pam) + 3) * 100 / len(self.ind_seq), 2)
                seq = rev_comp(seq)
                for i in range(len(self.seq_list)):
                    line = self.seq_list[i]
                    if line[1][2] != '+':
                        continue
                    cv = line[1][0] + self.rna_len - 3
                    distance = (pos + len(self.pam) + 3 + 1) - cv
                    if distance == 0:
                        distance -= 1
                    if self.nick_min <= abs(distance) <= self.nick_max:
                        sub_nick = [seq, pos + 1, rel_pos, '-', gc(seq[len(self.pam):]), distance, PE_type]
                        for x in range(self.mm_num):
                            sub_nick.append(0)
                        self.seq_list[i][4].append(sub_nick)
                        selected = True

            if selected == True:
                off_s = seq[:self.rna_len] + 'N' * len(self.pam) + ' ' + str(self.mm_num - 1)
                if off_s not in self.offinder_input:
                    self.offinder_input.append(off_s)
                    self.off_d[seq[:self.rna_len]] = []
                    for x in range(self.mm_num):
                        self.off_d[seq[:self.rna_len]].append(0)

            pos += 1
            m = pattern_r.search(self.ind_seq, pos)

    def run_cas_offinder(self):
        
        with open('cas-offinder-input.txt','w') as fw:
            fw.write('\n'.join(self.offinder_input))
        if self.use_cpus:
            proc = Popen(['cas-offinder', 'cas-offinder-input.txt', 'C', 'cas-offinder-output.txt'], stdout=PIPE).communicate()
        else:
            proc = Popen(['cas-offinder', 'cas-offinder-input.txt', 'G', 'cas-offinder-output.txt'], stdout=PIPE).communicate()

        with open('cas-offinder-output.txt') as f:
            for line in f:
                line_sp = line.strip().split('\t')
                self.off_d[line_sp[0][:self.rna_len]][int(line_sp[5])] += 1
        print(self.off_d)
            
    def write_result(self):
        
        multiply = 7 + self.mm_num
        menu = ["Target Sequence (5' to 3')", "Position", "Cleavage Position (%)", "Direction", "GC Content", "Edit Position", "PAM Change"]
        for i in range(self.mm_num): menu.append('Mismatch {0}'.format(i))
        menu += ["Type", "Extension Sequence", "PBS length", "PBS GC Content", "RTT length", "RTT GC Content", "Target Sequence (5' to 3')", "Position", "Cleavage Position (%)", "Direction", "GC Content", "Distance", "PE Type"]
        for i in range(self.mm_num): menu.append('Mimatch {0}'.format(i))
         
        with open('result.txt','w') as fw:
            fw.write('\t'.join(menu) + '\n')
            for i in self.seq_list:
                mm = self.off_d[i[0][:self.rna_len]]
                main_str = i[0] + '\t'
                for x in i[1]:
                    main_str += str(x) + '\t'
                for x in mm:
                    main_str += str(x) + '\t'
                for x in i[2]:
                    for y in i[3]:
                        if i[1][2] == '+':
                            s = main_str + 'pegRNA\t' + rev_comp(x[0] + ' ' + y[0]) + '\t' + str(x[1]) + '\t' + str(x[2]) + '\t' + str(y[1]) + '\t' + str(y[2]) + '\t' + '-\t'*multiply
                            fw.write(s[:-1] + '\n')
                        if i[1][2] == '-':
                            s = main_str + 'pegRNA\t' + y[0] + ' ' + x[0] + '\t' + str(x[1]) + '\t' + str(x[2]) + '\t' + str(y[1]) + '\t' + str(y[2]) + '\t' + '-\t'*multiply
                            fw.write(s[:-1] + '\n')
                for x in i[4]:
                    mm_nick = self.off_d[x[0][:self.rna_len]]
                    s = main_str + 'nCas9\t' + '-\t' * 5 + x[0] + '\t' + str(x[1]) + '\t' + str(x[2]) + '\t' + str(x[3]) + '\t' + str(x[4]) + '\t' + str(x[5]) + '\t' + str(x[6]) + '\t'
                    for y in mm_nick:
                        s += str(y) + '\t'
                    fw.write(s[:-1] + '\n')
        
def parse_args():
    
    parser = argparse.ArgumentParser()
    parser.add_argument("ref_directory", type=str, help="Path of reference sequence file in FASTA format")
    parser.add_argument("ind_directory", type=str, help="Path of reference sequence file in FASTA format")
    parser.add_argument("pam", type=str, help="PAM sequence")
    parser.add_argument("ref_dir", type=str, help="Directory of reference genome file, in FASTA or 2bit format")
    parser.add_argument("-l", type=int, help="length of target without PAM", default=20)
    parser.add_argument("-m", type=int, help="Mismatch number", default=2)
    parser.add_argument("--pbs_min", type=int, help="Minimum of PBS length", default = 12)
    parser.add_argument("--pbs_max", type=int, help="Maximum of PBS length", default = 14)
    parser.add_argument("--rtt_min", type=int, help="Minimum of RTT length", default = 10)
    parser.add_argument("--rtt_max", type=int, help="Maximum of RTT length", default = 20)
    parser.add_argument("--nick_min", type=int, help="Minimum of nicking distance", default = 0)
    parser.add_argument("--nick_max", type=int, help="Maximum of nicking distance", default = 100)
    parser.add_argument("--use_cpus", action='store_true', help="Use cpu instead of gpu (cas-offinder)", required=False)
    
    return parser.parse_args()


def main():
    
    tst = time.time()
    
    c = PeDesigner(parse_args())

    c.set_mutation()
    print('find target...')
    c.find_target()
    print('find off-target...')
    c.run_cas_offinder()
    print('writing results...')
    c.write_result()
    print('finish!')

if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pe-designer",
    version="2.0.0",
    author="Gue-ho",
    description="Prime editing tools that consist of a reverse transcriptase linked with Cas9 nickase are capable of generating targeted insertions, deletions,and base conversions without producing DNA double strand breaks or requiring any donor DNA.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Gue-ho/PE-Designer",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ),
)
